@@ -94,7 +94,7 @@ DEFAULTS = [
               'use_custom_margin': True,
               'custom_margin': 0,
               'use_custom_cursor_blinking': False,
-              'show_internal_console_if_traceback': False,
+              'show_internal_errors': True,
               'check_updates_on_startup': True,
               'toolbars_visible': True,
               # Global Spyder fonts
@@ -162,8 +162,7 @@ DEFAULTS = [
               'symbolic_math': False,
               'in_prompt': '',
               'out_prompt': '',
-              'light_color': True,
-              'dark_color': False
+              'show_elapsed_time': False
               }),
             ('variable_explorer',
              {
@@ -384,6 +383,8 @@ DEFAULTS = [
               'editor/go to line': 'Ctrl+L',
               'editor/go to previous file': 'Ctrl+Shift+Tab',
               'editor/go to next file': 'Ctrl+Tab',
+              'editor/cycle to previous file': 'Ctrl+PgUp',
+              'editor/cycle to next file': 'Ctrl+PgDown',
               'editor/new file': "Ctrl+N",
               'editor/open last closed':"Ctrl+Shift+T",
               'editor/open file': "Ctrl+O",
@@ -589,7 +590,7 @@ DEFAULTS = [
               'solarized/light/background':  '#fdf6e3',
               'solarized/light/currentline': '#f5efdB',
               'solarized/light/currentcell': '#eee8d5',
-              'solarized/light/occurence':   '#839496',
+              'solarized/light/occurrence':   '#839496',
               'solarized/light/ctrlclick':   '#d33682',
               'solarized/light/sideareas':   '#eee8d5',
               'solarized/light/matched_p':   '#586e75',
@@ -608,7 +609,7 @@ DEFAULTS = [
               'solarized/dark/background':  '#002b36',
               'solarized/dark/currentline': '#083f4d',
               'solarized/dark/currentcell': '#073642',
-              'solarized/dark/occurence':   '#657b83',
+              'solarized/dark/occurrence':   '#657b83',
               'solarized/dark/ctrlclick':   '#d33682',
               'solarized/dark/sideareas':   '#073642',
               'solarized/dark/matched_p':   '#93a1a1',
@@ -635,7 +636,7 @@ DEFAULTS = [
 #    or if you want to *rename* options, then you need to do a MAJOR update in
 #    version, e.g. from 3.0.0 to 4.0.0
 # 3. You don't need to touch this value if you're just adding a new option
-CONF_VERSION = '40.1.0'
+CONF_VERSION = '43.0.0'
 
 # Main configuration instance
 try:
