@@ -17,8 +17,6 @@ except ImportError:
     from mock import Mock, MagicMock  # Python 2
 
 # Third party imports
-from qtpy import PYQT_VERSION
-from qtpy.QtWidgets import QWidget
 from qtpy.QtWebEngineWidgets import WEBENGINE
 import pytest
 from flaky import flaky
@@ -34,17 +32,9 @@ from spyder.utils.introspection.utils import default_info_response
 @pytest.fixture
 def help_plugin(qtbot):
     """Help plugin fixture"""
-
-    class MainMock(QWidget):
-        def __getattr__(self, attr):
-            if attr == 'ipyconsole' or attr == 'editor':
-                return None
-            else:
-                return Mock()
-
-    help_plugin = Help(parent=MainMock())
-
+    help_plugin = Help()
     webview = help_plugin.rich_text.webview._webview
+
     if WEBENGINE:
         help_plugin._webpage = webview.page()
     else:
@@ -76,7 +66,6 @@ def check_text(widget, text):
 # Tests
 # =============================================================================
 @flaky(max_runs=3)
-@pytest.mark.skipif(PYQT_VERSION > '5.10', reason='Segfaults in PyQt 5.10+')
 def test_no_docs_message(help_plugin, qtbot):
     """
     Test that no docs message is shown when instrospection plugins
@@ -89,7 +78,6 @@ def test_no_docs_message(help_plugin, qtbot):
 
 
 @flaky(max_runs=3)
-@pytest.mark.skipif(PYQT_VERSION > '5.10', reason='Segfaults in PyQt 5.10+')
 def test_no_further_docs_message(help_plugin, qtbot):
     """
     Test that no further docs message is shown when instrospection
@@ -105,20 +93,26 @@ def test_no_further_docs_message(help_plugin, qtbot):
                     timeout=3000)
 
 
-@pytest.mark.skipif(PYQT_VERSION > '5.10', reason='Segfaults in PyQt 5.10+')
-def test_help_opens_when_show_tutorial_unit(help_plugin, qtbot):
+def test_help_opens_when_show_tutorial_unit(help_plugin, qtbot,):
     """Test fix for #6317 : 'Show tutorial' opens the help plugin if closed."""
     MockDockwidget = MagicMock()
     MockDockwidget.return_value.isVisible.return_value = False
     mockDockwidget_instance = MockDockwidget()
+
+    MockAction = Mock()
+    mock_toggle_view_action = MockAction()
     mock_show_rich_text = Mock()
 
     help_plugin.dockwidget = mockDockwidget_instance
+    help_plugin.toggle_view_action = mock_toggle_view_action
     help_plugin.show_rich_text = mock_show_rich_text
 
     help_plugin.show_tutorial()
     qtbot.wait(100)
 
+    assert mockDockwidget_instance.show.call_count == 1
+    assert mock_toggle_view_action.setChecked.call_count == 1
+    mock_toggle_view_action.setChecked.assert_called_once_with(True)
     assert mock_show_rich_text.call_count == 1
 
     MockDockwidget.return_value.isVisible.return_value = True
@@ -127,6 +121,8 @@ def test_help_opens_when_show_tutorial_unit(help_plugin, qtbot):
 
     help_plugin.show_tutorial()
     qtbot.wait(100)
+    assert mockDockwidget_instance.show.call_count == 1
+    assert mock_toggle_view_action.setChecked.call_count == 1
     assert mock_show_rich_text.call_count == 2
 
 
