@@ -20,7 +20,7 @@ import errno, os
 from time import time as _uniquefloat
 
 import psutil
-from spyder.config.base import running_under_pytest
+from spyder.config.base import PYTEST
 from spyder.py3compat import PY2, to_binary_string
 
 def unique():
@@ -42,7 +42,7 @@ else:
     import ctypes
     from ctypes import wintypes
 
-    # https://docs.microsoft.com/en-us/windows/desktop/ProcThread/process-security-and-access-rights
+    # http://msdn.microsoft.com/en-us/library/windows/desktop/ms684880(v=vs.85).aspx
     PROCESS_QUERY_INFORMATION = 0x400
 
     # GetExitCodeProcess uses a special exit code to indicate that the
@@ -50,7 +50,7 @@ else:
     STILL_ACTIVE = 259
     
     def _is_pid_running(pid):
-        """Taken from https://www.madebuild.org/blog/?p=30"""
+        """Taken from http://www.madebuild.org/blog/?p=30"""
         kernel32 = ctypes.windll.kernel32
         handle = kernel32.OpenProcess(PROCESS_QUERY_INFORMATION, 0, pid)
         if handle == 0:
@@ -184,10 +184,8 @@ class FilesystemLock:
                         p = psutil.Process(int(pid))
 
                         # Valid names for main script
-                        names = set(['spyder', 'spyder3', 'spyder.exe',
-                                     'spyder3.exe', 'bootstrap.py',
-                                     'spyder-script.py'])
-                        if running_under_pytest():
+                        names = set(['spyder', 'spyder3', 'bootstrap.py'])
+                        if PYTEST:
                             names.add('runtests.py')
 
                         # Check the first three command line arguments
