@@ -17,7 +17,7 @@ import re
 # Third party imports
 from qtpy.QtCore import Qt, QTimer, Signal, Slot, QEvent
 from qtpy.QtGui import QTextCursor
-from qtpy.QtWidgets import (QCheckBox, QGridLayout, QHBoxLayout, QLabel,
+from qtpy.QtWidgets import (QGridLayout, QHBoxLayout, QLabel,
                             QSizePolicy, QWidget)
 
 # Local imports
@@ -42,7 +42,14 @@ class FindReplace(QWidget):
     """Find widget"""
     STYLE = {False: "background-color:rgb(255, 175, 90);",
              True: "",
-             None: ""}
+             None: "",
+             'regexp_error': "background-color:rgb(255, 80, 80);",
+             }
+    TOOLTIP = {False: _("No matches"),
+               True: _("Search string"),
+               None: _("Search string"),
+               'regexp_error': _("Regular expression error")
+               }
     visibility_changed = Signal(bool)
     return_shift_pressed = Signal()
     return_pressed = Signal()
@@ -77,7 +84,8 @@ class FindReplace(QWidget):
 
         self.search_text.lineEdit().textEdited.connect(
                                                      self.text_has_been_edited)
-        
+
+        self.number_matches_text = QLabel(self)
         self.previous_button = create_toolbutton(self,
                                              triggered=self.find_previous,
                                              icon=ima.icon('ArrowUp'))
@@ -112,9 +120,9 @@ class FindReplace(QWidget):
 
         hlayout = QHBoxLayout()
         self.widgets = [self.close_button, self.search_text,
-                        self.previous_button, self.next_button,
-                        self.re_button, self.case_button, self.words_button,
-                        self.highlight_button]
+                        self.number_matches_text, self.previous_button,
+                        self.next_button, self.re_button, self.case_button,
+                        self.words_button, self.highlight_button]
         for widget in self.widgets[1:]:
             hlayout.addWidget(widget)
         glayout.addLayout(hlayout, 0, 1)
@@ -168,14 +176,14 @@ class FindReplace(QWidget):
         self.highlight_timer.timeout.connect(self.highlight_matches)
         self.search_text.installEventFilter(self)
 
-
     def eventFilter(self, widget, event):
         """Event filter for search_text widget.
 
         Emits signals when presing Enter and Shift+Enter.
         This signals are used for search forward and backward.
+        Also, a crude hack to get tab working in the Find/Replace boxes.
         """
-        if (event.type() == QEvent.KeyPress):
+        if event.type() == QEvent.KeyPress:
             key = event.key()
             shift = event.modifiers() & Qt.ShiftModifier
 
@@ -185,8 +193,13 @@ class FindReplace(QWidget):
                 else:
                     self.return_pressed.emit()
 
-        return super(FindReplace, self).eventFilter(widget, event)
+            if key == Qt.Key_Tab:
+                if self.search_text.hasFocus():
+                    self.replace_text.set_current_text(
+                        self.search_text.currentText())
+                self.focusNextChild()
 
+        return super(FindReplace, self).eventFilter(widget, event)
 
     def create_shortcuts(self, parent):
         """Create shortcuts for this widget"""
@@ -244,6 +257,7 @@ class FindReplace(QWidget):
         """Overrides Qt Method"""
         QWidget.show(self)
         self.visibility_changed.emit(True)
+        self.change_number_matches()
         if self.editor is not None:
             if hide_replace:
                 if self.replace_widgets[0].isVisible():
@@ -373,6 +387,16 @@ class FindReplace(QWidget):
         # When several lines are selected in the editor and replace box is activated, 
         # dynamic search is deactivated to prevent changing the selection. Otherwise
         # we show matching items.
+        def regexp_error_msg(pattern):
+            """Returns None if the pattern is a valid regular expression or
+            a string describing why the pattern is invalid.
+            """
+            try:
+                re.compile(pattern)
+            except re.error as e:
+                return str(e)
+            return None
+
         if multiline_replace_check and self.replace_widgets[0].isVisible() and \
            len(to_text_string(self.editor.get_selected_text()).splitlines())>1:
             return None
@@ -382,6 +406,7 @@ class FindReplace(QWidget):
             if not self.is_code_editor:
                 # Clears the selection for WebEngine
                 self.editor.find_text('')
+            self.change_number_matches()
             return None
         else:
             case = self.case_button.isChecked()
@@ -389,7 +414,17 @@ class FindReplace(QWidget):
             regexp = self.re_button.isChecked()
             found = self.editor.find_text(text, changed, forward, case=case,
                                           words=words, regexp=regexp)
-            self.search_text.lineEdit().setStyleSheet(self.STYLE[found])
+
+            stylesheet = self.STYLE[found]
+            tooltip = self.TOOLTIP[found]
+            if not found and regexp:
+                error_msg = regexp_error_msg(text)
+                if error_msg:  # special styling for regexp errors
+                    stylesheet = self.STYLE['regexp_error']
+                    tooltip = self.TOOLTIP['regexp_error'] + ': ' + error_msg
+            self.search_text.lineEdit().setStyleSheet(stylesheet)
+            self.search_text.setToolTip(tooltip)
+
             if self.is_code_editor and found:
                 if rehighlight or not self.editor.found_results:
                     self.highlight_timer.stop()
@@ -399,6 +434,14 @@ class FindReplace(QWidget):
                         self.highlight_matches()
             else:
                 self.clear_matches()
+
+            number_matches = self.editor.get_number_matches(text, case=case)
+            if hasattr(self.editor, 'get_match_number'):
+                match_number = self.editor.get_match_number(text, case=case)
+            else:
+                match_number = 0
+            self.change_number_matches(current_match=match_number,
+                                       total_matches=number_matches)
             return found
 
     @Slot()
@@ -407,7 +450,12 @@ class FindReplace(QWidget):
         if (self.editor is not None):
             replace_text = to_text_string(self.replace_text.currentText())
             search_text = to_text_string(self.search_text.currentText())
-            pattern = search_text if self.re_button.isChecked() else None
+            re_pattern = None
+            if self.re_button.isChecked():
+                try:
+                    re_pattern = re.compile(search_text)
+                except re.error:
+                    return  # do nothing with an invalid regexp
             case = self.case_button.isChecked()
             first = True
             cursor = None
@@ -417,7 +465,7 @@ class FindReplace(QWidget):
                     seltxt = to_text_string(self.editor.get_selected_text())
                     cmptxt1 = search_text if case else search_text.lower()
                     cmptxt2 = seltxt if case else seltxt.lower()
-                    if not pattern:
+                    if re_pattern is None:
                         has_selected = self.editor.has_selected_text()
                         if has_selected and cmptxt1 == cmptxt2:
                             # Text was already found, do nothing
@@ -427,7 +475,7 @@ class FindReplace(QWidget):
                                              rehighlight=False):
                                 break
                     else:
-                        if len(re.findall(pattern, cmptxt2)) > 0:
+                        if len(re_pattern.findall(cmptxt2)) > 0:
                             pass
                         else:
                             if not self.find(changed=False, forward=True,
@@ -457,13 +505,13 @@ class FindReplace(QWidget):
                         # Avoid infinite loop: single found occurrence
                         break
                     position0 = position1
-                if pattern is None:
+                if re_pattern is None:
                     cursor.removeSelectedText()
                     cursor.insertText(replace_text)
                 else:
                     seltxt = to_text_string(cursor.selectedText())
                     cursor.removeSelectedText()
-                    cursor.insertText(re.sub(pattern, replace_text, seltxt))
+                    cursor.insertText(re_pattern.sub(replace_text, seltxt))
                 if self.find_next():
                     found_cursor = self.editor.textCursor()
                     cursor.setPosition(found_cursor.selectionStart(),
@@ -488,36 +536,49 @@ class FindReplace(QWidget):
     @Slot()
     def replace_find_selection(self, focus_replace_text=False):
         """Replace and find in the current selection"""
-        if (self.editor is not None):
+        if self.editor is not None:
             replace_text = to_text_string(self.replace_text.currentText())
             search_text = to_text_string(self.search_text.currentText())
-            pattern = search_text if self.re_button.isChecked() else None
             case = self.case_button.isChecked()
             words = self.words_button.isChecked()
             re_flags = re.MULTILINE if case else re.IGNORECASE|re.MULTILINE
 
-            cursor = self.editor.textCursor()
-            cursor.beginEditBlock()
-            seltxt = to_text_string(self.editor.get_selected_text())
-            if not pattern:
+            re_pattern = None
+            if self.re_button.isChecked():
+                pattern = search_text
+            else:
                 pattern = re.escape(search_text)
                 replace_text = re.escape(replace_text)
-            if words:
-                #If whole words is checked we need to check that each match
-                #is actually a whole word before replacing
-                try:
-                    re.compile(pattern)
-                except re.error:
-                    return #if the pattern won't compile cancel the find/replace
-                word_pattern = r'\b{pattern}\b'.format(pattern = pattern)
-                replacement = re.sub(word_pattern, replace_text, seltxt, flags=re_flags)
-            else:
-                replacement = re.sub(pattern, replace_text, seltxt, flags=re_flags)
-            if replacement != seltxt:
+            if words:  # match whole words only
+                pattern = r'\b{pattern}\b'.format(pattern=pattern)
+            try:
+                re_pattern = re.compile(pattern, flags=re_flags)
+            except re.error as e:
+                return  # do nothing with an invalid regexp
+
+            selected_text = to_text_string(self.editor.get_selected_text())
+            replacement = re_pattern.sub(replace_text, selected_text)
+            if replacement != selected_text:
+                cursor = self.editor.textCursor()
+                cursor.beginEditBlock()
                 cursor.removeSelectedText()
+                if not self.re_button.isChecked():
+                    replacement = re.sub(r'\\(?![nrtf])(.)', r'\1', replacement)
                 cursor.insertText(replacement)
-            cursor.endEditBlock()
+                cursor.endEditBlock()
             if focus_replace_text:
                 self.replace_text.setFocus()
             else:
                 self.editor.setFocus()
+
+    def change_number_matches(self, current_match=0, total_matches=0):
+        """Change number of match and total matches."""
+        if current_match and total_matches:
+            matches_string = u"{} {} {}".format(current_match, _(u"of"),
+                                               total_matches)
+            self.number_matches_text.setText(matches_string)
+        elif total_matches:
+            matches_string = u"{} {}".format(total_matches, _(u"matches"))
+            self.number_matches_text.setText(matches_string)
+        else:
+            self.number_matches_text.setText(_(u"no matches"))
