@@ -51,6 +51,9 @@ def insert_text_to(cursor, text, fmt):
 
 class CompletionWidget(QListWidget):
     """Completion list widget"""
+
+    sig_show_completions = Signal(object)
+
     def __init__(self, parent, ancestor):
         QListWidget.__init__(self, ancestor)
         self.setWindowFlags(Qt.SubWindow | Qt.FramelessWindowHint)
@@ -146,6 +149,9 @@ class CompletionWidget(QListWidget):
             # to update the displayed list:
             self.update_current()
         
+        # signal used for testing
+        self.sig_show_completions.emit(completion_list)
+
     def hide(self):
         QListWidget.hide(self)
         self.textedit.setFocus()
@@ -599,11 +605,11 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
                 lines.pop(0)
             else:
                 break
-        
-        # Add an EOL character after indentation blocks that start with some 
+
+        # Add an EOL character after indentation blocks that start with some
         # Python reserved words, so that it gets evaluated automatically
         # by the console
-        varname = re.compile('[a-zA-Z0-9_]*') # matches valid variable names
+        varname = re.compile(r'[a-zA-Z0-9_]*')  # Matches valid variable names.
         maybe = False
         nextexcept = ()
         for n, line in enumerate(lines):
@@ -886,61 +892,70 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         cursor = self.textCursor()
         cursor.beginEditBlock()
         start_pos, end_pos = self.__save_selection()
-        add_linesep = False
-        if to_text_string(cursor.selectedText()):
-            # Check if start_pos is at the start of a block
-            cursor.setPosition(start_pos)
-            cursor.movePosition(QTextCursor.StartOfBlock)
-            start_pos = cursor.position()
-
-            cursor.setPosition(end_pos)
-            # Check if end_pos is at the start of a block: if so, starting
-            # changes from the previous block
-            cursor.movePosition(QTextCursor.StartOfBlock,
-                                QTextCursor.KeepAnchor)
-            if to_text_string(cursor.selectedText()):
-                cursor.movePosition(QTextCursor.NextBlock)
-                end_pos = cursor.position()
-        else:
-            cursor.movePosition(QTextCursor.StartOfBlock)
-            start_pos = cursor.position()
-            cursor.movePosition(QTextCursor.NextBlock)
-            end_pos = cursor.position()
-            # check if on last line
-            if end_pos == start_pos:
-                cursor.movePosition(QTextCursor.End)
-                end_pos = cursor.position()
-                if start_pos == end_pos:
-                    cursor.endEditBlock()
-                    return
-                add_linesep = True
+        last_line = False
+        
+        # ------ Select text
+        
+        # Get selection start location
         cursor.setPosition(start_pos)
-        cursor.setPosition(end_pos, QTextCursor.KeepAnchor)
-
-        sel_text = to_text_string(cursor.selectedText())
-        if add_linesep:
-            sel_text += os.linesep
-        cursor.removeSelectedText()
-
-        if after_current_line:
-            text = to_text_string(cursor.block().text())
-            if len(text) == 0:
-                #If the next line is blank
-                sel_text = sel_text[0:-1]
-                sel_text = os.linesep + sel_text
-            if not text:
-                cursor.insertText(sel_text)
-                cursor.endEditBlock()
-                return
-            start_pos += len(text)+1
-            end_pos += len(text)+1
+        cursor.movePosition(QTextCursor.StartOfBlock)
+        start_pos = cursor.position()
+        
+        # Get selection end location
+        cursor.setPosition(end_pos)
+        if not cursor.atBlockStart() or end_pos == start_pos:
+            cursor.movePosition(QTextCursor.EndOfBlock)
             cursor.movePosition(QTextCursor.NextBlock)
-            if cursor.position() < start_pos:
-                cursor.movePosition(QTextCursor.End)
-                sel_text = os.linesep + sel_text
-                end_pos -= 1
+        end_pos = cursor.position()
+        
+        # Check if selection ends on the last line of the document
+        if cursor.atEnd():
+            if not cursor.atBlockStart() or end_pos == start_pos:
+                last_line = True
+                
+        # ------ Stop if at document boundary
+        
+        cursor.setPosition(start_pos)
+        if cursor.atStart() and not after_current_line:
+            # Stop if selection is already at top of the file while moving up
+            cursor.endEditBlock()
+            self.setTextCursor(cursor)
+            self.__restore_selection(start_pos, end_pos)
+            return
+                
+        cursor.setPosition(end_pos, QTextCursor.KeepAnchor)
+        if last_line and after_current_line:
+            # Stop if selection is already at end of the file while moving down
+            cursor.endEditBlock()
+            self.setTextCursor(cursor)
+            self.__restore_selection(start_pos, end_pos)
+            return
+        
+        # ------ Move text
+        
+        sel_text = to_text_string(cursor.selectedText())
+        cursor.removeSelectedText()
+        
+        
+        if after_current_line:
+            # Shift selection down
+            text = to_text_string(cursor.block().text())  
+            sel_text = os.linesep + sel_text[0:-1]  # Move linesep at the start
+            cursor.movePosition(QTextCursor.EndOfBlock)
+            start_pos += len(text)+1
+            end_pos += len(text)
+            if not cursor.atEnd():
+                end_pos += 1        
         else:
-            cursor.movePosition(QTextCursor.PreviousBlock)
+            # Shift selection up
+            if last_line:
+                # Remove the last linesep and add it to the selected text
+                cursor.deletePreviousChar()
+                sel_text = sel_text + os.linesep
+                cursor.movePosition(QTextCursor.StartOfBlock)
+                end_pos += 1
+            else:
+                cursor.movePosition(QTextCursor.PreviousBlock)
             text = to_text_string(cursor.block().text())
             start_pos -= len(text)+1
             end_pos -= len(text)+1
@@ -993,6 +1008,31 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         cursor.endEditBlock()
         self.ensureCursorVisible()
 
+    def set_selection(self, start, end):
+        cursor = self.textCursor()
+        cursor.setPosition(start)
+        cursor.setPosition(end, QTextCursor.KeepAnchor)
+        self.setTextCursor(cursor)
+
+    def truncate_selection(self, position_from):
+        """Unselect read-only parts in shell, like prompt"""
+        position_from = self.get_position(position_from)
+        cursor = self.textCursor()
+        start, end = cursor.selectionStart(), cursor.selectionEnd()
+        if start < end:
+            start = max([position_from, start])
+        else:
+            end = max([position_from, end])
+        self.set_selection(start, end)
+
+    def restrict_cursor_position(self, position_from, position_to):
+        """In shell, avoid editing text except between prompt and EOF"""
+        position_from = self.get_position(position_from)
+        position_to = self.get_position(position_to)
+        cursor = self.textCursor()
+        cursor_position = cursor.position()
+        if cursor_position < position_from or cursor_position > position_to:
+            self.set_cursor_position(position_to)
 
     #------Code completion / Calltips
     def hide_tooltip_if_necessary(self, key):
@@ -1272,7 +1312,7 @@ class ConsoleFontStyle(object):
 class ConsoleBaseWidget(TextEditBaseWidget):
     """Console base widget"""
     BRACE_MATCHING_SCOPE = ('sol', 'eol')
-    COLOR_PATTERN = re.compile('\x01?\x1b\[(.*?)m\x02?')
+    COLOR_PATTERN = re.compile(r'\x01?\x1b\[(.*?)m\x02?')
     exception_occurred = Signal(str, bool)
     userListActivated = Signal(int, str)
     completion_widget_activated = Signal(str)
@@ -1320,32 +1360,6 @@ class ConsoleBaseWidget(TextEditBaseWidget):
                              foreground=QColor(Qt.lightGray))
         self.ansi_handler.set_light_background(state)
         self.set_pythonshell_font()
-        
-    def set_selection(self, start, end):
-        cursor = self.textCursor()
-        cursor.setPosition(start)
-        cursor.setPosition(end, QTextCursor.KeepAnchor)
-        self.setTextCursor(cursor)
-
-    def truncate_selection(self, position_from):
-        """Unselect read-only parts in shell, like prompt"""
-        position_from = self.get_position(position_from)
-        cursor = self.textCursor()
-        start, end = cursor.selectionStart(), cursor.selectionEnd()
-        if start < end:
-            start = max([position_from, start])
-        else:
-            end = max([position_from, end])
-        self.set_selection(start, end)
-
-    def restrict_cursor_position(self, position_from, position_to):
-        """In shell, avoid editing text except between prompt and EOF"""
-        position_from = self.get_position(position_from)
-        position_to = self.get_position(position_to)
-        cursor = self.textCursor()
-        cursor_position = cursor.position()
-        if cursor_position < position_from or cursor_position > position_to:
-            self.set_cursor_position(position_to)
 
     #------Python shell
     def insert_text(self, text):
