@@ -24,7 +24,7 @@ def setup_browser(qtbot):
 def test_browser(qtbot):
     """Run web browser."""
     browser = setup_browser(qtbot)
-    browser.set_home_url('https://www.google.com/')
+    browser.set_home_url('http://www.google.com/')
     browser.go_home()
     browser.show()
     assert browser
