@@ -1,25 +1,22 @@
 # -*- coding: utf-8 -*-
-# -----------------------------------------------------------------------------
-# Copyright © Spyder Project Contributors
 #
+# Copyright © Spyder Project Contributors
 # Licensed under the terms of the MIT License
-# (see spyder/__init__.py for details)
-# -----------------------------------------------------------------------------
 
 """
-Tests for the dataframe editor.
+Tests for dataframeeditor.py
 """
 
 from __future__ import division
 
 # Standard library imports
-import os
 from sys import platform
 from datetime import datetime
 try:
     from unittest.mock import Mock, ANY
 except ImportError:
     from mock import Mock, ANY  # Python 2
+import os
 
 # Third party imports
 from pandas import DataFrame, date_range, read_csv, concat
@@ -38,16 +35,9 @@ from spyder.widgets.variableexplorer.dataframeeditor import (
     DataFrameEditor, DataFrameModel)
 from spyder.py3compat import PY2
 
-
-# =============================================================================
-# Constants
-# =============================================================================
 FILES_PATH = os.path.dirname(os.path.realpath(__file__))
 
-
-# =============================================================================
-# Utility functions
-# =============================================================================
+# Helper functions
 def colorclose(color, hsva_expected):
     """
     Compares HSV values which are stored as 16-bit integers.
@@ -61,10 +51,9 @@ def data(dfm, i, j):
 def bgcolor(dfm, i, j):
     return dfm.get_bgcolor(dfm.createIndex(i, j))
 
+# --- Tests
+# -----------------------------------------------------------------------------
 
-# =============================================================================
-# Tests
-# =============================================================================
 def test_dataframemodel_basic():
     df = DataFrame({'colA': [1, 3], 'colB': ['c', 'a']})
     dfm = DataFrameModel(df)
@@ -177,7 +166,7 @@ def test_dataframemodel_with_format_percent_d_and_nan():
     """
     Test DataFrameModel with format `%d` and dataframe containing NaN
 
-    Regression test for issue #4139.
+    Regression test for issue 4139.
     """
     np_array = numpy.zeros(2)
     np_array[1] = numpy.nan
@@ -286,11 +275,7 @@ def test_sort_dataframe_with_category_dtypes(qtbot):  # cf. issue 5361
 
 
 def test_dataframemodel_set_data_overflow(monkeypatch):
-    """
-    Test that entry of an overflowing integer is caught and handled properly.
-
-    Unit regression test for issue #6114 .
-    """
+    """Test for #6114: Overflowing ints are caught and handled properly"""
     MockQMessageBox = Mock()
     attr_to_patch = ('spyder.widgets.variableexplorer' +
                      '.dataframeeditor.QMessageBox')
@@ -318,11 +303,7 @@ def test_dataframemodel_set_data_overflow(monkeypatch):
 @flaky(max_runs=3)
 @pytest.mark.no_xvfb
 def test_dataframeeditor_edit_overflow(qtbot, monkeypatch):
-    """
-    Test that entry of an overflowing integer is caught and handled properly.
-
-    Integration regression test for issue #6114 .
-    """
+    """Test #6114: Entry of an overflow int is caught and handled properly"""
     MockQMessageBox = Mock()
     attr_to_patch = ('spyder.widgets.variableexplorer' +
                      '.dataframeeditor.QMessageBox')
@@ -345,32 +326,26 @@ def test_dataframeeditor_edit_overflow(qtbot, monkeypatch):
         qtbot.waitForWindowShown(dialog)
         view = dialog.dataTable
 
-        qtbot.keyClick(view, Qt.Key_Right)
+        qtbot.keyPress(view, Qt.Key_Right)
         qtbot.keyClicks(view, '5')
-        qtbot.keyClick(view, Qt.Key_Down)
-        qtbot.keyClick(view, Qt.Key_Space)
-        qtbot.keyClick(view.focusWidget(), Qt.Key_Backspace)
+        qtbot.keyPress(view, Qt.Key_Down)
+        qtbot.keyPress(view, Qt.Key_Space)
+        qtbot.keyPress(view.focusWidget(), Qt.Key_Backspace)
         qtbot.keyClicks(view.focusWidget(), str(int(2 ** bit_exponet)))
-        qtbot.keyClick(view.focusWidget(), Qt.Key_Down)
+        qtbot.keyPress(view.focusWidget(), Qt.Key_Down)
         MockQMessageBox.critical.assert_called_with(ANY, "Error", ANY)
         assert MockQMessageBox.critical.call_count == idx
         qtbot.keyClicks(view, '7')
-        qtbot.keyClick(view, Qt.Key_Up)
+        qtbot.keyPress(view, Qt.Key_Up)
         qtbot.keyClicks(view, '6')
-        qtbot.keyClick(view, Qt.Key_Down)
-        qtbot.wait(200)
-        dialog.accept()
-        qtbot.wait(500)
+        qtbot.keyPress(view, Qt.Key_Down)
+        qtbot.keyPress(view, Qt.Key_Return)
         assert numpy.sum(expected_df[0].as_matrix() ==
                          dialog.get_value().as_matrix()) == len(expected_df)
 
 
 def test_dataframemodel_set_data_complex(monkeypatch):
-    """
-    Test that editing complex dtypes is handled gracefully in df editor.
-
-    Unit regression test for issue #6115 .
-    """
+    """Test for #6115: Editing complex dtypes raises error in df editor"""
     MockQMessageBox = Mock()
     attr_to_patch = ('spyder.widgets.variableexplorer' +
                      '.dataframeeditor.QMessageBox')
@@ -392,11 +367,7 @@ def test_dataframemodel_set_data_complex(monkeypatch):
 @flaky(max_runs=3)
 @pytest.mark.no_xvfb
 def test_dataframeeditor_edit_complex(qtbot, monkeypatch):
-    """
-    Test that editing complex dtypes is handled gracefully in df editor.
-
-    Integration regression test for issue #6115 .
-    """
+    """Test for #6115: editing complex dtypes raises error in df editor"""
     MockQMessageBox = Mock()
     attr_to_patch = ('spyder.widgets.variableexplorer' +
                      '.dataframeeditor.QMessageBox')
@@ -412,30 +383,29 @@ def test_dataframeeditor_edit_complex(qtbot, monkeypatch):
         qtbot.waitForWindowShown(dialog)
         view = dialog.dataTable
 
-        qtbot.keyClick(view, Qt.Key_Right)
-        qtbot.keyClick(view, Qt.Key_Down)
-        qtbot.keyClick(view, Qt.Key_Space)
-        qtbot.keyClick(view.focusWidget(), Qt.Key_Backspace)
+        qtbot.keyPress(view, Qt.Key_Right)
+        qtbot.keyPress(view, Qt.Key_Down)
+        qtbot.keyPress(view, Qt.Key_Space)
+        qtbot.keyPress(view.focusWidget(), Qt.Key_Backspace)
         qtbot.keyClicks(view.focusWidget(), "42")
-        qtbot.keyClick(view.focusWidget(), Qt.Key_Down)
+        qtbot.keyPress(view.focusWidget(), Qt.Key_Down)
         MockQMessageBox.critical.assert_called_with(ANY, "Error", ANY)
         assert MockQMessageBox.critical.call_count == count * 2 - 1
-        qtbot.keyClick(view, Qt.Key_Down)
+        qtbot.keyPress(view, Qt.Key_Down)
         qtbot.keyClick(view, '1')
-        qtbot.keyClick(view.focusWidget(), Qt.Key_Down)
+        qtbot.keyPress(view.focusWidget(), Qt.Key_Down)
         MockQMessageBox.critical.assert_called_with(
             ANY, "Error", ("Editing dtype {0!s} not yet supported."
                            .format(type(test_df.iloc[1, 0]).__name__)))
         assert MockQMessageBox.critical.call_count == count * 2
-        qtbot.wait(200)
-        dialog.accept()
-        qtbot.wait(500)
+        qtbot.keyPress(view, Qt.Key_Return)
+        qtbot.wait(1000)
         assert numpy.sum(test_df[0].as_matrix() ==
                          dialog.get_value().as_matrix()) == len(test_df)
 
 
 def test_dataframemodel_set_data_bool(monkeypatch):
-    """Test that bools are editible in df and false-y strs are detected."""
+    """Test that bools are editible in df and false-y strs are detected"""
     MockQMessageBox = Mock()
     attr_to_patch = ('spyder.widgets.variableexplorer' +
                      '.dataframeeditor.QMessageBox')
@@ -458,7 +428,7 @@ def test_dataframemodel_set_data_bool(monkeypatch):
 @flaky(max_runs=3)
 @pytest.mark.no_xvfb
 def test_dataframeeditor_edit_bool(qtbot, monkeypatch):
-    """Test that bools are editible in df and false-y strs are detected."""
+    """Test that bools are editible in df and false-y strs are detected"""
     MockQMessageBox = Mock()
     attr_to_patch = ('spyder.widgets.variableexplorer' +
                      '.dataframeeditor.QMessageBox')
@@ -476,16 +446,15 @@ def test_dataframeeditor_edit_bool(qtbot, monkeypatch):
         qtbot.waitForWindowShown(dialog)
         view = dialog.dataTable
 
-        qtbot.keyClick(view, Qt.Key_Right)
+        qtbot.keyPress(view, Qt.Key_Right)
         for test_str in test_strs:
-            qtbot.keyClick(view, Qt.Key_Space)
-            qtbot.keyClick(view.focusWidget(), Qt.Key_Backspace)
+            qtbot.keyPress(view, Qt.Key_Space)
+            qtbot.keyPress(view.focusWidget(), Qt.Key_Backspace)
             qtbot.keyClicks(view.focusWidget(), test_str)
-            qtbot.keyClick(view.focusWidget(), Qt.Key_Down)
+            qtbot.keyPress(view.focusWidget(), Qt.Key_Down)
             assert not MockQMessageBox.critical.called
-        qtbot.wait(200)
-        dialog.accept()
-        qtbot.wait(500)
+        qtbot.keyPress(view, Qt.Key_Return)
+        qtbot.wait(1000)
         assert (numpy.sum(expected_df[0].as_matrix() ==
                           dialog.get_value().as_matrix()[:, 0]) ==
                 len(expected_df))
